# Models for PowerschoolPy
from dataclasses import dataclass
from typing import Any, Dict, List
from attr import dataclass

@dataclass
class Student:
    """ Defines a student """
    studentId: int
    studentDcid: int
    firstName: str
    middleName: str
    lastName: str
    gradeLevel: int
    currentMealBalance: float
    currentGPA: float
    courses: List

    @staticmethod
    def from_dict(data: Dict[str, Any]):
        """ Return a student object from PowerSchool """
        return Student(
            studentId = data["studentId"],
            studentDcid = data["studentDcid"],
            firstName = data["student"]["firstName"],
            middleName = data["student"]["middleName"],
            lastName = data["student"]["lastName"],
            gradeLevel = data["student"]["gradeLevel"],
            currentMealBalance = data["student"]["currentMealBalance"],
            currentGPA = data["student"]["currentGPA"],
            courses = []
        )

@dataclass
class Course:
    """ Defines a course """
    id: int
    dcid: int
    courseName: str
    courseCode: str
    roomNumber: int
    period: int
    teacherId: int

    @staticmethod
    def from_dict(data: Dict[str, Any]):
        """ Return a student object from PowerSchool """
        return Course(
            id = data["sections"]["id"],
            dcid = data["sections"]["dcid"],
            courseName = data["sections"]["schoolCourseTitle"],
            courseCode = data["sections"]["courseCode"],
            roomNumber = data["sections"]["roomName"],
            period = data["sections"]["periodSort"],
            teacherId = data["sections"]["teacherID"]
        )

# @dataclass
# class Assignment:
#     """ Defines an assignment"""
#     assignmentID: int
#     name: str
#     grade: str
#     percent: str
#     isMissing : bool
#     wasLate: bool
#     duedate: str

#     def from_dict(data: Dict[str, Any]):
#         """ Return a student object from PowerSchool """
#         return Assignment(
#             assignmentID = data["assignmentScores"]["assignmentId"],
#             name = data["assignmentScores"]["name"],
#             grade = data["assignmentScores"]["grade"],
#             percent = data["assignmentScores"]["percent"],
#             isMissing = data["assignmentScores"]["isMissing"],
#             wasLate = data["assignmentScores"]["wasLate"],
#             duedate = data["assignmentScores"]["duedate"]
#         )

    # 'finalGrades': [
    #     {
    #         'commentValue': 'Works diligently and cares to complete work to the best quality. Shows positive character.',
    #         'dateStored': None,
    #         'grade': 'A',
    #         'id': 546481,
    #         'percent': 100.0,
    #         'reportingTermId': 1002352,
    #         'sectionid': 48013,
    #         'storeType': 2
    #     },


            # assignments
    # 'assignmentScores': [
    #     {
    #         'assignmentId': 140279,
    #         'collected': False,
    #         'comment': None,
    #         'exempt': False,
    #         'gradeBookType': 2,
    #         'id': 2023294,
    #         'incomplete': False,
    #         'late': False,
    #         'letterGrade': 'A',
    #         'missing': False,
    #         'percent': '100.0',
    #         'score': '3.0',
    #         'scoretype': 0
    #     },

    #   'assignments': [
    #     {
    #         'abbreviation': None,
    #         'additionalCategoryIds': [],
    #         'assignmentid': 111692,
    #         'categoryId': 743,
    #         'description': None,
    #         'dueDate': datetime.datetime(2022, 9, 16, 7, 0, tzinfo=<isodate.tzinfo.Utc object at 0x0000022CC98C27A0>),
    #         'gradeBookType': 2,
    #         'id': 140279,
    #         'includeinfinalgrades': 1,
    #         'name': 'Daily Health #1',
    #         'pointspossible': 3.0,
    #         'publishDaysBeforeDue': 0,
    #         'publishState': 0,
    #         'publishonspecificdate': datetime.datetime(2022, 11, 4, 7, 0, tzinfo=<isodate.tzinfo.Utc object at 0x0000022CC98C27A0>),
    #         'publishscores': 1,
    #         'sectionDcid': 46758,
    #         'sectionid': 47884,
    #         'type': 0,
    #         'weight': 1.0
    #     },      


#!/usr/bin/env python3
import zeep, requests, logging
import zeep.helpers
from .exceptions import PowerschoolPyError
from .models import Student, Course


class PowerschoolPy:
    """The client for connecting to powerschool

    Client(base_url, api_username="pearson", api_password="m0bApP5")

    A client for using powerschool methods"""

    def __init__(self, host, api_username="pearson", api_password="m0bApP5"):
        self._api_username = api_username
        self._api_password = api_password
        self._host = host
        # logging.basicConfig(level=logging.CRITICAL)
        # session = requests.session()
        # session.auth = requests.auth.HTTPDigestAuth(api_username, api_password)
        # if host[:-1] != "/":
        #     host += "/"
        # self.url = host + "pearson-rest/services/PublicPortalServiceJSON"
        # try:
        #     self.client = zeep.Client(
        #         wsdl=self.url + "?wsdl",
        #         transport=zeep.transports.Transport(session=session),
        #     )
        # except requests.exceptions.ConnectionError:
        #     raise PowerschoolPyError(f"Could not connect to {host}.")
        # except requests.exceptions.HTTPError:
        #     raise PowerschoolPyError(
        #         f"Incorrect api credentials {api_username}, {api_password}"
        #     )

    async def login(self):
        logging.basicConfig(level=logging.CRITICAL)
        session = requests.session()
        session.auth = requests.auth.HTTPDigestAuth(
            self._api_username, self._api_password
        )
        if self._host[:-1] != "/":
            self._host += "/"
        self._url = self._host + "pearson-rest/services/PublicPortalServiceJSON"
        try:
            self.client = zeep.Client(
                wsdl=self._url + "?wsdl",
                transport=zeep.transports.Transport(session=session),
            )
        except requests.exceptions.ConnectionError:
            raise PowerschoolPyError(f"Could not connect to {host}.")
        except requests.exceptions.HTTPError:
            raise PowerschoolPyError(
                f"Incorrect api credentials {api_username}, {api_password}"
            )
        return True

    async def getStudent(self, username, password, studentIDToFind=None):
        studentCounter = 0

        service = self.client.create_service(
            "{http://publicportal.rest.powerschool.pearson.com}PublicPortalServiceJSONSoap12Binding",
            self._url,
        )
        result = service.loginToPublicPortal(username, password)["userSessionVO"]

        if result["userId"] == None:
            raise PowerschoolPyError(
                "Could not log in to ({}, {})".format(username, password)
            )

        userSessionVO = {
            "userId": result["userId"],
            "serviceTicket": result["serviceTicket"],
            "serverInfo": {"apiVersion": result["serverInfo"]["apiVersion"]},
            "serverCurrentTime": result["serverCurrentTime"],
            "userType": result["userType"],
        }

        for studentID in result["studentIDs"]:
            if studentIDToFind == None:
                student = Student.from_dict(
                    service.getStudentData(
                        userSessionVO, result["studentIDs"][0], {"includes": "1"}
                    )["studentDataVOs"][0]
                )
                # sections = result["studentIDs"][0]["sections"]
                # for course in sections:
                #     student.courses.append(Course.from_dict(course))
            elif studentID == studentIDToFind:
                student = service.getStudentData(
                    userSessionVO,
                    result["studentIDs"][studentCounter],
                    {"includes": "1"},
                )["studentDataVOs"][0]
                # for course in result["studentIDs"][studentCounter]["sections"]:
                #     student.courses.append(Course.from_dict(course))

            studentCounter = studentCounter + 1

            # for course in result["sections"]:
            #     courses.append(Course.from_dict(course))

        # return Student.from_dict(student)
        return student

    async def getStudents(self, username, password):
        students = []
        studentCounter = 0

        service = self.client.create_service(
            "{http://publicportal.rest.powerschool.pearson.com}PublicPortalServiceJSONSoap12Binding",
            self._url,
        )
        result = service.loginToPublicPortal(username, password)["userSessionVO"]

        if result["userId"] == None:
            raise PowerschoolPyError(
                "Could not log in to ({}, {})".format(username, password)
            )

        userSessionVO = {
            "userId": result["userId"],
            "serviceTicket": result["serviceTicket"],
            "serverInfo": {"apiVersion": result["serverInfo"]["apiVersion"]},
            "serverCurrentTime": result["serverCurrentTime"],
            "userType": result["userType"],
        }

        for studentID in result["studentIDs"]:
            # students.append(service.getStudentData(userSessionVO, result["studentIDs"][studentCounter], {"includes": "1"})["studentDataVOs"][0])
            student = Student.from_dict(
                service.getStudentData(
                    userSessionVO,
                    result["studentIDs"][studentCounter],
                    {"includes": "1"},
                )["studentDataVOs"][0]
            )
            students.append(student)
            studentCounter = studentCounter + 1

        return students



from os import read, system
import os
import pathlib
import re
import sys
from setuptools import setup, find_packages

def get_version():
    """Get current version from code."""
    regex = r"__version__\s=\s\"(?P<version>[\d\.]+?)\""
    path = ("PowerSchoolPy", "__version__.py")
    return re.search(regex, read(*path)).group("version")

def read(*parts):
    """Read file."""
    filename = os.path.join(os.path.abspath(os.path.dirname(__file__)), *parts)
    sys.stdout.write(filename)
    with open(filename, encoding="utf-8", mode="rt") as fp:
        return fp.read()

with open("README.md") as readme_file:
    readme = readme_file.read()


setup(
    author="Brian Lich",
    author_email="blich29@hotmail.com",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: AsyncIO",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    description="An async library for Powerschool",
    include_package_data=True,
    install_requires=["aiohttp>=3.0.0"],
    keywords=["powerschool", "grades", "async", "client"],
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
    name="PowerSchoolPy",
    packages=find_packages(include=["PowerSchoolPy"]),
    url="https://github.com/brianlich/PowerSchoolPy",
    version=get_version(),
    zip_safe=False,
)